#!/usr/bin/python
# 3.5
"""
pypinner.cli

Ensure requirements.txt files reflect versions in pip freeze.
Finds requirements*.txt files recursively and fixes them.

Usage:
  pypinner
  pypinner -f <frozen_requirements.txt>
  pypinner (-h | --help)
  pypinner --version

Options:
  -h --help     Show this screen.
  --version     Show version.

"""

import fileinput
import glob
import logging
import subprocess
import sys

import pypinner
from docopt import docopt

logging.basicConfig(
    level=logging.INFO,
    format='%(relativeCreated)6d %(threadName)s %(message)s'
)
LOG = logging.getLogger()


def find_files():
    """
        Find requirements files recursively from PWD.
        Searches for requirements*.txt
    """
    return glob.glob("./**/requirements*.txt", recursive=True)


def req_to_dict(req):
    """
        Naive split on == in requirements file.
        Also, only really interested in hard pinned requirements.
    """
    return dict([dep.split('==') for dep in req.splitlines() if '==' in dep])


def get_pip_freeze():
    """
        Run pip freeze and return requirements as a dict.
    """
    result = subprocess.run(['pip', 'freeze'], stdout=subprocess.PIPE)
    req = result.stdout.decode()
    return req_to_dict(req)


def get_frozen_requirements(file):
    """
        Load requirements from a file and return as a dict.
    """
    with open(file, "r") as f:
        return req_to_dict(f.read())


def run(arguments):
    """
        Run the pypinner with the supplied arguments.
    """
    files = find_files()
    if arguments['-f']:
        frozen = get_frozen_requirements(arguments['<frozen_requirements.txt>'])
    else:
        frozen = get_pip_freeze()
    with fileinput.input(files=files, inplace=True, backup=".bak") as requirements:
        for req in requirements:
            req = req.strip()
            if req in frozen:
                req = "{}=={}".format(req, frozen[req])
                sys.stderr.write("Pinning: {}\t{}\n".format(fileinput.filename(), req))
            else:
                sys.stderr.write("Leaving: {}\t{}\n".format(fileinput.filename(), req))
            print(req)
    return 0


def main():
    """
        Main entrypoint.
    """
    arguments = docopt(__doc__, version=pypinner.__version__)
    exit(run(arguments))



import os
import sys

from setuptools import find_packages, setup
from setuptools.command.test import test as TestCommand

"""
    Setup.py
    Required files:
        README.md
        {PROJECT}/version.txt
        MANIFEST.in
        requirements.txt
        requirements_test.txt
"""

# Mandatory project/module name
PROJECT = 'pypinner'
KEYWORDS = "python pip freeze version pinner"
URL = ""
# Optional entrypoint {PROJECT}.cli:main()
CLI = 'pypinner'


class PyTest(TestCommand):

    def initialize_options(self):
        TestCommand.initialize_options(self)
        self.pytest_args = [
            '-s',
            '-ra',
            '--verbose',
            '--flake8',
            '--isort',
            '--cov-report=xml',
            '--cov-report=term-missing',
            '--cov={}'.format(PROJECT),
            '--junitxml=junit.xml',
        ]

    def finalize_options(self):
        TestCommand.finalize_options(self)
        self.test_args = []
        self.test_suite = True

    def run_tests(self):
        import pytest
        errno = pytest.main(self.pytest_args)
        sys.exit(errno)


def slurp(file):
    return open(os.path.join(os.path.abspath(os.path.dirname(__file__)), file)).read().strip()


REQUIREMENTS = slurp('requirements.txt').splitlines()
REQUIREMENTS_TEST = ['setuptools']
REQUIREMENTS_TEST.extend(slurp('requirements_test.txt').splitlines())
REQUIREMENTS_TEST.extend(REQUIREMENTS)

setup_cfg = {
    'name': PROJECT,
    'version': slurp(os.path.join(PROJECT, "version.txt")),
    'license': "MIT",
    'description': slurp('README.md').splitlines()[3],
    'long_description': slurp('README.md'),
    'url': URL,
    'keywords': KEYWORDS,
    'packages': find_packages(),
    'py_modules': [PROJECT],
    'cmdclass': {'test': PyTest},
    'tests_require': REQUIREMENTS_TEST,
    'install_requires': REQUIREMENTS,
    'dependency_links': [],
    'package_data': {
        '': ['*.txt', '*.md', '*.json'],
    },
    'classifiers': [
        # Development Status: 3 - Alpha, 4 - Beta, 5 - Production/Stable
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Developers',
        'Natural Language :: English',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.6',
        'Topic :: Software Development :: Libraries :: Python Modules',
    ],
}

if CLI:
    setup_cfg['entry_points'] = {
        'console_scripts': [
            '{} = {}.cli:main'.format(CLI, PROJECT)
        ]
    }

setup(**setup_cfg)
